"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const listr_1 = __importDefault(require("listr"));
const Environment = __importStar(require("../../utils/environment-tasks"));
const Process = __importStar(require("../../utils/process"));
const Runner = __importStar(require("../../utils/runner"));
const version_1 = __importDefault(require("../../version"));
const orchestrator_1 = __importDefault(require("@mesg/orchestrator"));
const execution_1 = require("@mesg/orchestrator/lib/execution");
const api_1 = __importDefault(require("@mesg/api"));
const base58 = __importStar(require("@mesg/api/lib/util/base58"));
const chalk_1 = __importDefault(require("chalk"));
const encoder_1 = require("@mesg/orchestrator/lib/encoder");
const sign_1 = __importDefault(require("../../utils/sign"));
const ipfsClient = require('ipfs-http-client');
class Dev extends command_1.Command {
    constructor() {
        super(...arguments);
        this.lcdEndpoint = 'http://localhost:1317';
        this.orchestratorEndpoint = 'localhost:50052';
        this.lcd = new api_1.default(this.lcdEndpoint);
        this.orchestrator = new orchestrator_1.default(this.orchestratorEndpoint);
        this.ipfsClient = ipfsClient('ipfs.app.mesg.com', '5001', { protocol: 'http' });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(Dev);
            let compilation;
            let deployedProcess;
            const tasks = new listr_1.default([
                Environment.start,
                {
                    title: 'Compiling process',
                    task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                        compilation = yield Process.compile(args.PROCESS_FILE, this.ipfsClient, this.lcd, this.lcdEndpoint, this.orchestratorEndpoint, ctx.config.mnemonic, ctx.engineAddress, flags.env);
                    })
                },
                {
                    title: 'Creating process',
                    task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                        deployedProcess = yield Process.create(this.lcd, compilation.definition, ctx.config.mnemonic);
                    })
                },
                {
                    title: 'Fetching process\'s logs',
                    task: ctx => {
                        const payload = {
                            filter: {
                                statuses: [
                                    execution_1.Status.Completed,
                                    execution_1.Status.Failed
                                ]
                            }
                        };
                        this.logs = this.orchestrator.execution.stream(payload, sign_1.default(payload, ctx.config.mnemonic));
                    }
                }
            ]);
            const { config, engineAddress } = yield tasks.run({
                configDir: this.config.dataDir,
                pull: flags.pull,
                version: flags.version,
                endpoint: this.lcdEndpoint,
            });
            this.logs
                .on('error', (error) => { this.warn('Result stream error: ' + error.message); })
                .on('data', (execution) => {
                if (!execution.processHash)
                    return;
                if (!execution.instanceHash)
                    return;
                if (base58.encode(execution.processHash) !== deployedProcess.hash)
                    return;
                const prefix = `[${execution.nodeKey}] - ${base58.encode(execution.instanceHash)} - ${execution.taskKey}`;
                if (execution.error) {
                    this.log(`${prefix}: ` + chalk_1.default.red('ERROR:', execution.error));
                }
                if (!execution.outputs)
                    return;
                return this.log(prefix +
                    '\n\tinputs:  ' + chalk_1.default.gray(JSON.stringify(encoder_1.decode(execution.inputs || {}))) +
                    '\n\toutputs: ' + chalk_1.default.gray(JSON.stringify(encoder_1.decode(execution.outputs || {}))) +
                    '\n');
            });
            process.once('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
                yield new listr_1.default([
                    {
                        title: 'Stopping logs',
                        task: () => {
                            if (this.logs)
                                this.logs.cancel();
                        }
                    },
                    {
                        title: 'Deleting process',
                        task: () => __awaiter(this, void 0, void 0, function* () {
                            if (deployedProcess)
                                yield Process.remove(this.lcd, deployedProcess, config.mnemonic);
                        })
                    },
                    {
                        title: 'Stopping services',
                        task: () => __awaiter(this, void 0, void 0, function* () {
                            for (const runner of compilation.runners) {
                                yield Runner.stop(this.lcdEndpoint, this.orchestratorEndpoint, config.mnemonic, engineAddress, runner.hash);
                            }
                        })
                    },
                    Environment.stop
                ]).run({
                    configDir: this.config.dataDir
                });
            }));
        });
    }
}
exports.default = Dev;
Dev.description = 'Run a process in a local development environment';
Dev.flags = {
    version: command_1.flags.string({ name: 'Engine version', default: version_1.default.engine }),
    pull: command_1.flags.boolean({ name: 'Force to pull the docker image', default: false }),
    env: command_1.flags.string({
        description: 'Environment variables to inject to the process',
        multiple: true,
        helpValue: 'FOO=BAR'
    })
};
Dev.args = [{
        name: 'PROCESS_FILE',
        description: 'Path of a process file'
    }];
