"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const listr_1 = __importDefault(require("listr"));
const fs_1 = require("fs");
const path_1 = require("path");
const docker_1 = require("./docker");
const config_1 = require("./config");
const node_fetch_1 = __importDefault(require("node-fetch"));
const api_1 = __importDefault(require("@mesg/api"));
const pidFilename = 'pid.json';
const image = 'mesg/engine';
const pids = (path) => fs_1.existsSync(path_1.join(path, pidFilename))
    ? JSON.parse(fs_1.readFileSync(path_1.join(path, pidFilename)).toString()).map((x) => parseInt(x, 10))
    : [];
const appendPID = (path) => {
    const res = [...pids(path), process.pid].filter((value, index, self) => self.indexOf(value) === index);
    fs_1.writeFileSync(path_1.join(path, pidFilename), JSON.stringify(res));
};
const removePID = (path) => {
    if (!fs_1.existsSync(path_1.join(path, pidFilename)))
        return;
    const res = pids(path).filter(x => x !== process.pid);
    fs_1.writeFileSync(path_1.join(path, pidFilename), JSON.stringify(res));
};
const hasOtherInstances = (path) => pids(path).filter(x => x !== process.pid).length > 0;
const isRunning = (endpoint = `http://localhost:1317`) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        yield node_fetch_1.default(`${endpoint}/node_info`);
        const { block } = yield (yield node_fetch_1.default(`${endpoint}/blocks/latest`)).json();
        return parseInt(block.header.height, 10) > 0;
    }
    catch (e) {
        return false;
    }
});
const waitToBeReady = (endpoint) => __awaiter(void 0, void 0, void 0, function* () {
    if (yield isRunning(endpoint))
        return;
    return new Promise(resolve => {
        setTimeout(() => waitToBeReady().then(resolve), 1000);
    });
});
exports.stop = {
    title: 'Stopping environment',
    task: () => new listr_1.default([
        {
            title: 'Stopping the Engine',
            skip: ctx => hasOtherInstances(ctx.configDir),
            task: () => __awaiter(void 0, void 0, void 0, function* () {
                const containers = yield docker_1.listContainers({ label: [docker_1.engineLabel] });
                if (containers.length === 0)
                    throw new Error('Cannot find engine');
                const container = containers[0];
                yield container.stop();
                yield container.delete();
                const network = yield docker_1.findNetwork(docker_1.engineName);
                if (network)
                    yield network.remove();
            }),
        },
        {
            title: 'Clearing config',
            skip: ctx => hasOtherInstances(ctx.configDir),
            task: ctx => config_1.clear(ctx.configDir)
        },
        {
            title: 'Unregistering CLI',
            task: ctx => removePID(ctx.configDir)
        },
    ])
};
exports.start = {
    title: 'Starting environment',
    task: () => new listr_1.default([
        {
            title: 'Creating default configuration',
            skip: ctx => fs_1.existsSync(ctx.configDir),
            task: ctx => fs_1.mkdirSync(ctx.configDir, { recursive: true })
        },
        {
            title: 'Registering CLI',
            task: ctx => appendPID(ctx.configDir)
        },
        {
            title: 'Generating test account',
            task: ctx => ctx.config = config_1.generateConfig(ctx.configDir)
        },
        {
            title: 'Updating the Engine image',
            skip: (ctx) => __awaiter(void 0, void 0, void 0, function* () { return !ctx.pull && (yield docker_1.hasImage(`${image}:${ctx.version}`)); }),
            task: ctx => docker_1.fetchImageTag(image, ctx.version)
        },
        {
            title: 'Starting the Engine',
            skip: (ctx) => __awaiter(void 0, void 0, void 0, function* () { return (yield isRunning(ctx.endpoint)) || (yield docker_1.listContainers({ label: [docker_1.engineLabel] })).length > 0; }),
            task: ctx => docker_1.createContainer(`${image}:${ctx.version}`, ctx.configDir)
        },
        {
            title: 'Waiting for the Engine to be ready',
            skip: ctx => isRunning(ctx.endpoint),
            task: (ctx) => __awaiter(void 0, void 0, void 0, function* () { return waitToBeReady(ctx.endpoint); })
        },
        {
            title: 'Ensuring balance',
            task: (ctx) => __awaiter(void 0, void 0, void 0, function* () {
                const api = new api_1.default(ctx.endpoint);
                const engineAccount = yield api.account.import(ctx.config.engine.account.mnemonic);
                ctx.engineAddress = engineAccount.address;
                const userAccount = yield api.account.import(ctx.config.mnemonic);
                const userBalance = userAccount.coins.find(x => x.denom === 'atto');
                if (userBalance && parseInt(userBalance.amount, 10) > 1000)
                    return;
                const transferMsg = api.account.transferMsg(engineAccount.address, userAccount.address, [{
                        amount: 1e18.toString(),
                        denom: 'atto'
                    }]);
                const tx = yield api.createTransaction([transferMsg], engineAccount);
                yield api.broadcast(tx.signWithMnemonic(ctx.config.engine.account.mnemonic), "block");
            })
        }
    ])
};
